#!/usr/bin/perl -w

use strict;
use warnings;


my $usage = << "USE";
	
	Split large chromosomes.
	
	Usage: fasta splitter
	perl ngm_splitter.pl <genome fasta> <max size>
USE

my $argcount = $#ARGV + 1;
my $reqArgs = 2;
$argcount == $reqArgs or die "$usage";

my $file_path = $ARGV[0];
my $size = $ARGV[1] * 1000000;

open(FHI, "<$file_path") or die "Unable to open file $file_path \n";

my $current = 0;

my $current_id = "";
my $current_sequence = "";
my $current_position = 0;
my $current_size = 0;

my $chr_length = 0;

my $last_position = 0;

while(<FHI>) {

#print $_;
#	chomp($_);
	if(/^>/) {
		#Header line
		if(length($current_id) > 0) {
			#Not first chromosome
			if($current == 0) {
				#Chromosome < size
				#print STDERR $current_id . "\n";
				print $current_id . "\n";
				print $current_sequence;			
			} else {
				#Chromosome > size
				#print STDERR $current_id . "_$last_position " . $size . "\n";
				print $current_id . "_$last_position\n";
				print $current_sequence;
				$current = 0;					
			}
			print STDERR "\@SQ\tSN:" . substr($current_id, 1) . "\tLN:$chr_length\n";
			$chr_length = 0;
			#exit(0);
		}
		$current_id = $_;
		$current_position = 0;
		$last_position = 0;
		chomp($current_id);
		$current_sequence = "";
	} else {
		#Non-header line
		$current_sequence .= $_;
		$current_position += (length($_) - 1);
		$chr_length += (length($_) - 1);
		if(length($current_sequence) > $size) {
			#Split chromosome
			#print STDERR $current_id . "_$last_position " . $size . "\n";
			print $current_id . "_$last_position\n";
			print $current_sequence;
			$current_sequence = substr($current_sequence, -1000, 1000);
			#print STDERR length($current_sequence) . "\n";
			$current += 1;	
			my $correction = lc($current_sequence) =~ tr/\n//;
			$last_position = $current_position - (1000 - $correction);					
		}
	}
	

}
if($current == 0) {
	#print STDERR $current_id . "\n";
	print $current_id . "\n";
	print $current_sequence;			
} else {
	#print STDERR $current_id . "_$last_position " . $size . "\n";
	print $current_id . "_$last_position\n";
	print $current_sequence;
	$current = 0;					
}
print STDERR "\@SQ     SN:" . substr($current_id, 1) . " LN:$chr_length\n";
close(FHI);

